import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

/** Longest Collatz Sequence.
 *
 * The Collatz Sequence is a hailstone algorithm, that is defined by applying
 * the following two rules to n to create the next n in the sequence:
 *
 * n => n/2 (n is even)
 * n => 3n+1 (n is odd)
 *
 * Example sequence:
 * 13 -> 40 -> 20 -> 10 -> 5 -> 16 -> 8 -> 4 -> 2 -> 1
 *
 * What is the starting number under one million that produces
 * the longest chain?
 *
 *
 * Right now, this throws a stack overflow error after 100,000.
 * The sequence length gets to be 350 or so, and then it craps out.
 * We can probably prime the hash table:
 * for the first 1,000 or so, we just add to the hash table.
 * Then we switch to hash table mode.
 */

public class Problem014 implements EulerSolution {

	// Compute longest Collatz sequence for integers up to MAX
	public static final int LOOPMAX = 1000000;
	public static final int ARRMAX = LOOPMAX;

	public String run() { 
		int maxLength = 0;
		int maxN = 0;
		int[] collatz = new int[ARRMAX];
		for(int i=0; i<LOOPMAX; i++) { 

			// For each number, only interested in 
			// the number of terms it takes to reach 1.
			//
			// For number i, this is stored at collatz[i]
			//
			// If collatz[i] is not zero, we have seen this number before.
			// Otherwise, walk thru sequence until you find a number
			// you have seen before.
			//
			// i = 0-based (the index)
			// j = i+1 = 1-based (the integer)
			int j = i+1;
			int length = computeCollatzLength(j,collatz);
			if(length>maxLength) { 
				maxLength = length;
				maxN = j;
			}
		}

		return new Integer(maxN).toString();
	}



	/** Computes the number of terms in the Collatz sequence until integer n reaches 1. */
	public static int computeCollatzLength(int n, int[] collatz) { 
		if(n<1) { throw new IllegalArgumentException(); }
		if(n==1) { return 1; }

		int lencount = 1; // this is what we will return
		long j = n; // integer
		int i = n-1; // index

		// easy case
		if(collatz[i]>0) {
			// we have seen integer n already
			lencount = collatz[i];
			return lencount;
		}

		// Save numbers encountered during this sequence to update collatz
		LinkedList<Integer> sequence = new LinkedList<Integer>();
		sequence.add((int)j);

		// Keep going until we reach 1, or short-circuit
		while(j!=1) { 

			long priorj = j;
			// Apply even/odd rule to get next j, and advance length of sequence by 1
			j = (j%2==0)?even(j):odd(j);


			//if(j<0) {
			//	System.out.println("j < 0: n = " + n + " prior j = " + priorj);
			//}

			if(j<ARRMAX) { 
				// Add new j to numbers encountered
				sequence.add((int)j);

				i = (int)(j-1);
				// Check if we have already memoized new j
				if(j<ARRMAX && collatz[i] > 0) { 
					// Short circuit 
					lencount += collatz[i];
					break;
				}
			} else {
				sequence.add(0); // dummy value
			}
			// Goes after memoization so we don't double-count
			lencount++;

		}

		// Before we return, update collatz memo 
		// with all numbers encountered
		int count = 0;
		for(Integer k : sequence) { 
			int km1 = k-1;
			if(km1<ARRMAX && km1>0) {
				// The first integer encountered is our integer,
				// and the sequence length is lencount.
				// Subsequent integers have a sequence length
				// of lencount - count
				collatz[km1] = lencount - count;
			}
			// This needs to go on the outside, of course.
			count++;
		}
		return lencount;
	}

	public static long even(long n) { return n/2; }
	public static long odd(long n) { return 3*n+1; }

}
