import java.util.ArrayList;
import java.util.Arrays;

/**
 * Finds the least number N for which the proportion of bouncy numbers
 * (neither increasing nor decreasing) is exactly 99%.
 * This is equivalent to finding the least N such that the count of
 * non-bouncy numbers NB(N) satisfies NB(N) = N / 100.
 * N must be a multiple of 100.
 */
public class Problem112 {

    // Memoization tables for Digit DP
    // memoInc[index][prevDigit][isLessThan][isLeadingZero]
    static long[][][][] memoInc;
    // memoDec[index][memoPrevDigit][isLessThan][isLeadingZero] (memoPrevDigit 0-9, 10 for initial state)
    static long[][][][] memoDec;
    // Digits of the number n being processed
    static ArrayList<Integer> digits;

    /**
     * Converts a number n into a list of its digits.
     */
    static void prepareDigits(long n) {
        digits = new ArrayList<>();
        String s = Long.toString(n);
        for (char c : s.toCharArray()) {
            digits.add(c - '0');
        }
        if (digits.isEmpty() && n == 0) digits.add(0); // Handle n=0 if it arises
    }

    // --- Digit DP Core for Increasing Numbers ---
    // Counts numbers x <= n such that x is increasing (non-decreasing digits). Includes 0.
    static long dfsInc(int index, int prevDigit, boolean isLessThan, boolean isLeadingZero) {
        if (index == digits.size()) {
            return 1; // Successfully formed a number
        }
        // Check memoization table
        if (memoInc[index][prevDigit][isLessThan ? 1 : 0][isLeadingZero ? 1 : 0] != -1) {
            return memoInc[index][prevDigit][isLessThan ? 1 : 0][isLeadingZero ? 1 : 0];
        }

        long count = 0;
        // Determine the upper bound for the current digit
        int upperBound = isLessThan ? 9 : digits.get(index);

        for (int digit = 0; digit <= upperBound; digit++) {
            // Determine the 'isLessThan' flag for the next recursive call
            boolean nextIsLessThan = isLessThan || (digit < upperBound);

            if (isLeadingZero) {
                if (digit == 0) {
                    // Still leading zeros, pass prevDigit=0 (doesn't matter yet)
                    count += dfsInc(index + 1, 0, nextIsLessThan, true);
                } else {
                    // First non-zero digit, start checking increasing property
                    count += dfsInc(index + 1, digit, nextIsLessThan, false);
                }
            } else {
                // Not a leading zero, must satisfy increasing condition
                if (digit >= prevDigit) {
                    count += dfsInc(index + 1, digit, nextIsLessThan, false);
                }
            }
        }

        // Store result in memoization table
        memoInc[index][prevDigit][isLessThan ? 1 : 0][isLeadingZero ? 1 : 0] = count;
        return count;
    }

    // --- Digit DP Core for Decreasing Numbers ---
    // Counts numbers x <= n such that x is decreasing (non-increasing digits). Includes 0.
    static long dfsDec(int index, int prevDigit, boolean isLessThan, boolean isLeadingZero) {
        if (index == digits.size()) {
            return 1;
        }
        // Map the state (prevDigit 0-9, or initial state) to memo index (0-10)
        int memoPrev = isLeadingZero ? 10 : prevDigit;
        if (memoDec[index][memoPrev][isLessThan ? 1 : 0][isLeadingZero ? 1 : 0] != -1) {
            return memoDec[index][memoPrev][isLessThan ? 1 : 0][isLeadingZero ? 1 : 0];
        }

        long count = 0;
        int upperBound = isLessThan ? 9 : digits.get(index);

        for (int digit = 0; digit <= upperBound; digit++) {
            boolean nextIsLessThan = isLessThan || (digit < upperBound);

            if (isLeadingZero) {
                if (digit == 0) {
                    // Still leading zeros, pass marker prevDigit=10
                    count += dfsDec(index + 1, 10, nextIsLessThan, true);
                } else {
                    // First non-zero digit
                    count += dfsDec(index + 1, digit, nextIsLessThan, false);
                }
            } else {
                // Not a leading zero, must satisfy decreasing condition
                if (digit <= prevDigit) {
                    count += dfsDec(index + 1, digit, nextIsLessThan, false);
                }
            }
        }

        memoDec[index][memoPrev][isLessThan ? 1 : 0][isLeadingZero ? 1 : 0] = count;
        return count;
    }

    /**
     * Calculates the count of positive increasing numbers less than or equal to n.
     */
    static long countIncreasing(long n) {
        if (n < 1) return 0;
        prepareDigits(n);
        int len = digits.size();
        // Dimensions: index (0..len), prevDigit (0-9), isLessThan (0/1), isLeadingZero (0/1)
        memoInc = new long[len + 1][10][2][2];
        // *** CORRECTED Initialization Loop for 4D array ***
        for (long[][][] arr3D : memoInc) {
            for (long[][] arr2D : arr3D) {
                for (long[] arr1D : arr2D) {
                    Arrays.fill(arr1D, -1);
                }
            }
        }
        // Initial call result includes 0, subtract 1 for positive count.
        return dfsInc(0, 0, false, true) - 1;
    }

    /**
     * Calculates the count of positive decreasing numbers less than or equal to n.
     */
    static long countDecreasing(long n) {
        if (n < 1) return 0;
        prepareDigits(n);
        int len = digits.size();
        // Dimensions: index(0..len), memoPrevDigit(0-10), isLessThan(0/1), isLeadingZero(0/1)
        memoDec = new long[len + 1][11][2][2]; // Size 11 for 0-9 and initial state 10
        // *** CORRECTED Initialization Loop for 4D array ***
        for (long[][][] arr3D : memoDec) {
            for (long[][] arr2D : arr3D) {
                for (long[] arr1D : arr2D) {
                    Arrays.fill(arr1D, -1);
                }
            }
        }
        // Initial call result includes 0, subtract 1 for positive count.
        return dfsDec(0, 10, false, true) - 1;
    }

    /**
     * Calculates the count of positive "flat" numbers (all digits same, e.g., 7, 22, 999)
     * less than or equal to n.
     */
    static long countFlat(long n) {
        if (n <= 0) return 0;
        String s = Long.toString(n);
        int len = s.length();
        long count = 0;

        // Count flat numbers with fewer digits (1..9, 11..99, ..., up to len-1 digits)
        for (int k = 1; k < len; k++) {
            count += 9; // 11..1, 22..2, ..., 99..9
        }

        // Count flat numbers with exactly 'len' digits
        for (int d = 1; d <= 9; d++) {
            // Construct the flat number string (safer for large n near Long.MAX_VALUE)
            char[] flatChars = new char[len];
            Arrays.fill(flatChars, (char)('0' + d));
            String flatNumStr = new String(flatChars);

            // Compare the flat number string with n's string
            // Lexicographical comparison works correctly for positive numbers of the same length
            if (flatNumStr.compareTo(s) <= 0) {
                count++;
            }
        }
        return count;
    }

     /**
      * Calculates the count of positive non-bouncy numbers less than or equal to n.
      * NB(N) = I(N) + D(N) - F(N)
      */
    static long countNonBouncy(long n) {
        if (n <= 0) return 0;
        // Calculate counts using helper functions
        long increasingCount = countIncreasing(n);
        long decreasingCount = countDecreasing(n);
        long flatCount = countFlat(n); // Flat numbers are counted in both I(N) and D(N)

        // Apply Inclusion-Exclusion Principle
        return increasingCount + decreasingCount - flatCount;
    }

    // --- Main Search Logic ---
    public static void main(String[] args) {
        // Start searching from a number known to have <99% bouncy proportion
        // N=21780 -> 90%, N~1M -> 98.7%. Let's start around 1M.
        long n = 1_000_000;
        // Ensure N is a multiple of 100
        if (n % 100 != 0) {
            n = (n / 100 + 1) * 100;
        }

        System.out.println("Starting search for the least N such that NB(N) = N / 100...");
        long startTime = System.currentTimeMillis();

        while (true) {
            // Optional: Print progress periodically
            if (n % 500_000 == 0) {
                long currentTime = System.currentTimeMillis();
                System.out.printf("... Checking N = %,d (%.1fs elapsed)%n", n, (currentTime - startTime) / 1000.0);
            }

            // Calculate the non-bouncy count for the current N
            long nbCount = countNonBouncy(n);

            // Check the target condition: NB(N) == N / 100
            // Ensure N is positive and divisible by 100 before division
            if (n > 0 && n % 100 == 0 && nbCount == n / 100) {
                long endTime = System.currentTimeMillis();
                System.out.println("-----------------------------------------");
                System.out.printf("FOUND! Least N = %,d%n", n);
                System.out.printf("Non-bouncy count NB(N) = %,d%n", nbCount);
                System.out.printf("Verification: N / 100 = %,d%n", n / 100);
                double bouncyProportion = (double) (n - nbCount) * 100.0 / n;
                System.out.printf("Proportion Bouncy = %.5f%%%n", bouncyProportion);
                System.out.printf("Time taken: %.3f seconds%n", (endTime - startTime) / 1000.0);
                System.out.println("-----------------------------------------");
                break; // Exit the loop once the condition is met
            }

            // --- Safety Checks (Optional) ---
             // Example limit - adjust if solution is expected to be much larger
             if (n > 50_000_000) {
                 System.out.println("Search limit (N > 50,000,000) exceeded without finding solution.");
                 break;
             }
             // Timeout - stop if it runs too long
             if (System.currentTimeMillis() - startTime > 10 * 60 * 1000) { // 10 minutes timeout
                  System.out.println("Search timed out (> 10 minutes).");
                  break;
             }

            // Move to the next multiple of 100
            n += 100;
        }
    }
}
