import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class Problem312 {

    static final long MOD = 13L*13*13*13*13*13*13*13; // 13^8 = 302875106592253

    // Modular exponentiation (using BigInteger for safety with large modulus)
    public static long power(long base, long exp, long modulus) {
        BigInteger result = BigInteger.valueOf(base)
                                    .modPow(BigInteger.valueOf(exp), BigInteger.valueOf(modulus));
        return result.longValueExact(); // Assumes result fits in long, which it should for mod M
    }

    // The function f(x) = (27 * x^3) mod modulus, defining the recurrence for n > 3
    static long f(long x, long modulus) {
        long twentySevenModM = 27L % modulus;
        long xCubedModM = power(x, 3L, modulus);
        long result = (twentySevenModM * xCubedModM) % modulus;
        return (result < 0) ? result + modulus : result;
    }

    // --- Periodicity Finding ---

    static class PeriodInfo {
        long lambda = -1; // Pre-period length relative to START OF SEQUENCE (index 1)
        long mu = -1;     // Period length
        List<Long> sequence = new ArrayList<>(); // Stores C(1) mod M, C(2) mod M, ...
        Map<Long, Long> firstSeenAtIndex = new HashMap<>(); // value -> index k for C(k) mod M
    }

    // Finds cycle in the sequence C(k) mod modulus
    static PeriodInfo findCycle(long modulus) {
        PeriodInfo info = new PeriodInfo();

        // Add explicit base cases modulo M
        long c1 = 1L % modulus;
        long c2 = 1L % modulus;
        long c3 = 8L % modulus;

        info.sequence.add(c1); // Index 0 holds C(1)
        info.firstSeenAtIndex.put(c1, 1L);
        if (c1 == c2 && !info.firstSeenAtIndex.containsKey(c2)) { // Handle value repeating early
             info.firstSeenAtIndex.put(c2, 2L);
        } else if (!info.firstSeenAtIndex.containsKey(c2)){
            info.firstSeenAtIndex.put(c2, 2L);
        }
         info.sequence.add(c2); // Index 1 holds C(2)


         if (!info.firstSeenAtIndex.containsKey(c3)) {
             info.firstSeenAtIndex.put(c3, 3L);
         }
         info.sequence.add(c3); // Index 2 holds C(3)


        long currentIndex = 4; // Start calculating C(4)
        long currentVal = c3; // C(3) is needed to calculate C(4)

        while (true) {
            currentVal = f(currentVal, modulus); // Calculate C(currentIndex) mod M
             info.sequence.add(currentVal); // Add C(currentIndex) to list

            if (info.firstSeenAtIndex.containsKey(currentVal)) {
                // Cycle detected!
                long firstSeenIdx = info.firstSeenAtIndex.get(currentVal);
                info.lambda = firstSeenIdx - 1; // Pre-period length (number of terms before cycle starts)
                info.mu = currentIndex - firstSeenIdx; // Cycle length
                System.out.println("Cycle detected for modulus " + modulus);
                System.out.println("Value " + currentVal + " first seen at C(" + firstSeenIdx + ") repeated at C(" + currentIndex + ")");
                System.out.println("Lambda (pre-period length): " + info.lambda);
                System.out.println("Mu (period length): " + info.mu);
                break; // Exit loop
            } else {
                info.firstSeenAtIndex.put(currentVal, currentIndex);
            }

            currentIndex++;

            // Safety break - adjust limit as needed
            if (currentIndex > 2 * modulus + 5) {
                 System.err.println("Exceeded iteration limit - no cycle found? Modulus=" + modulus);
                 info.mu = -1; // Indicate failure
                 break;
            }
        }
        return info;
    }

    // Computes C(k) mod modulus using periodicity info
    static long computeC_mod(long k, PeriodInfo info, long modulus) {
        if (k <= 0) throw new IllegalArgumentException("k must be positive");

        if (info.mu <= 0) { // Check if cycle was found
             // If no cycle (or error finding it), try direct computation for small k
             if (k == 1) return 1L % modulus;
             if (k == 2) return 1L % modulus;
             if (k == 3) return 8L % modulus;
             // Fallback might require recalculating if k is beyond stored sequence and no cycle info exists
             System.err.println("Cannot compute C(" + k + ") mod " + modulus + ": Cycle info invalid.");
             return -1; // Indicate error
        }

        if (k <= info.lambda) {
            // k is in the pre-period (or exactly the start of cycle if lambda includes first cycle element)
            // Sequence list index is k-1
            if (k-1 >= info.sequence.size()) {
                 System.err.println("Error: k=" + k + " requested, but stored sequence only up to index " + (info.sequence.size()-1));
                 return -1; // Or throw exception
            }
            return info.sequence.get((int)(k - 1));
        } else {
            // k is in the periodic part
            // Find the equivalent index within the cycle part of the stored sequence
            // The cycle starts at index lambda in the list (corresponding to C(lambda+1))
            long indexInCycle = (k - (info.lambda + 1)) % info.mu;
            int listIndex = (int)(info.lambda + indexInCycle);
             if (listIndex >= info.sequence.size()) {
                  System.err.println("Error: k=" + k + " maps to list index " + listIndex + ", but stored sequence only up to index " + (info.sequence.size()-1));
                  return -1; // Or throw exception
             }
            return info.sequence.get(listIndex);
        }
    }


    public static void main(String[] args) {
        long N = 10000L;

        // --- Step 1: Find period for Modulus M = 13^8 ---
        System.out.println("Finding cycle for M = " + MOD);
        PeriodInfo infoM = findCycle(MOD);
        if (infoM.mu <= 0) {
            System.err.println("Failed to find cycle for M=" + MOD + ". Cannot proceed.");
            return;
        }
        long mu = infoM.mu; // The period length modulo M

        // --- Step 2: Compute C(N) mod mu ---
        // We need the period info for modulus mu
        System.out.println("\nFinding cycle for modulus mu = " + mu);
        PeriodInfo infoMu = findCycle(mu);
        if (infoMu.mu <= 0 && mu != 1) { // Handle mu=1 case where cycle length is 0 or 1 technically
             // If mu=1, C(k) mod 1 = 0 for all k (unless mu was calculated incorrectly)
             if (mu == 1) {
                 System.out.println("Period mu=1, C(k) mod 1 is 0.");
                 // Set dummy infoMu for computeC_mod maybe? Or handle directly.
             } else {
                System.err.println("Failed to find cycle for mu=" + mu + ". Cannot proceed reliably.");
                // We might need C(N) mod mu, cannot get it easily without cycle info for mu.
                // We *could* iterate 10000 times directly modulo mu...
                // return; // Let's try direct iteration for C(N) mod mu as fallback
             }
        }

        System.out.println("\nComputing C(" + N + ") mod " + mu);
        long X_mod_mu;
        if (infoMu.mu > 0) { // If cycle found for mu
             X_mod_mu = computeC_mod(N, infoMu, mu);
        } else if (mu == 1) {
            X_mod_mu = 0; // C(N) mod 1 is 0
        }
         else { // Fallback: Direct iteration if cycle finding failed for mu
             System.out.println("Fallback: Computing C(10000) mod mu by direct iteration...");
             long h_prev = 8L % mu; // C(3) mod mu
             long current_h = h_prev;
             for(long i = 4; i <= N; i++) {
                 current_h = f(h_prev, mu);
                 h_prev = current_h;
             }
              X_mod_mu = current_h; // C(10000) mod mu
              // Handle base cases < 4 properly if N<4
              if(N==1) X_mod_mu = 1L % mu;
              if(N==2) X_mod_mu = 1L % mu;
              if(N==3) X_mod_mu = 8L % mu;

             System.out.println("Fallback result: " + X_mod_mu);
         }
        System.out.println("C(" + N + ") mod " + mu + " = " + X_mod_mu);


        // --- Step 3: Compute C(X) mod M ---
        System.out.println("\nComputing C(X) mod " + MOD + " using X mod " + mu);
        // Assume C(N) = X is large enough to be in the periodic part (k >= lambda+1)
        // Calculate effective index k_X corresponding to X
        // (X - (lambda_M + 1)) mod mu_M
        long effectiveIndexInCycle_X = (X_mod_mu - (infoM.lambda + 1 % mu) + mu ) % mu; // Ensure positive result before final modulo
        long k_X = infoM.lambda + 1 + effectiveIndexInCycle_X; // Actual index k corresponding to value X
        System.out.println("Value C(N) corresponds to index k_X = " + k_X + " in the sequence mod M");

        long Y_mod_M = computeC_mod(k_X, infoM, MOD);
        System.out.println("C(X) mod " + MOD + " = " + Y_mod_M);

        // --- Step 4: Compute C(X) mod mu --- (Needed for next step)
         System.out.println("\nComputing C(X) mod " + mu);
         long Y_mod_mu;
         if (infoMu.mu > 0) { // Use periodicity if available for mu
             Y_mod_mu = computeC_mod(k_X, infoMu, mu);
         } else if (mu == 1) {
             Y_mod_mu = 0;
         } else { // Fallback: Direct iteration mod mu up to index k_X
             System.out.println("Fallback: Computing C(" + k_X + ") mod mu by direct iteration...");
             long h_prev = 8L % mu; // C(3) mod mu
             long current_h = h_prev;
              if (k_X <= 3) { // Handle base cases for k_X
                  if(k_X==1) current_h = 1L % mu;
                  else if(k_X==2) current_h = 1L % mu;
                  else if(k_X==3) current_h = 8L % mu;
              } else {
                 for(long i = 4; i <= k_X; i++) {
                     current_h = f(h_prev, mu);
                     h_prev = current_h;
                 }
              }
             Y_mod_mu = current_h;
             System.out.println("Fallback result: " + Y_mod_mu);
         }
        System.out.println("C(X) mod " + mu + " = " + Y_mod_mu);


        // --- Step 5: Compute C(Y) mod M ---
        System.out.println("\nComputing C(Y) mod " + MOD + " using Y mod " + mu);
         // Assume C(X) = Y is large enough to be in the periodic part (k >= lambda+1)
         long effectiveIndexInCycle_Y = (Y_mod_mu - (infoM.lambda + 1 % mu) + mu ) % mu;
         long k_Y = infoM.lambda + 1 + effectiveIndexInCycle_Y;
         System.out.println("Value C(X) corresponds to index k_Y = " + k_Y + " in the sequence mod M");

        long Z_mod_M = computeC_mod(k_Y, infoM, MOD);
        System.out.println("C(Y) mod " + MOD + " = " + Z_mod_M);


        System.out.println("\nFinal Answer: C(C(C(" + N + "))) mod " + MOD + " = " + Z_mod_M);
    }
}
