import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

///////////////////////////////////////////
// Multithreaded version
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
// End multithreaded version
///////////////////////////////////////////

public class Problem932 {
    
    static class Result implements Comparable<Result> {
        BigInteger number;
        BigInteger a;
        BigInteger b;
        int sumVal;
        
        Result(BigInteger number, BigInteger a, BigInteger b, int sumVal) {
            this.number = number;
            this.a = a;
            this.b = b;
            this.sumVal = sumVal;
        }
        
        @Override
        public int compareTo(Result other) {
            return this.number.compareTo(other.number);
        }
    }
    
    /**
     * Find all 2025-numbers generated from sums in the range [startSum, endSum]
     * where the resulting number has at most n digits.
     */
    public static List<Result> find2025NumbersInRange(int startSum, int endSum, int n) {
        List<Result> foundNumbers = new ArrayList<>();
        
        // Pre-compute powers of 10 for efficiency
        BigInteger[] powersOf10 = new BigInteger[n + 1];
        for (int i = 0; i <= n; i++) {
            powersOf10[i] = BigInteger.TEN.pow(i);
        }
        
        for (int sumVal = startSum; sumVal <= endSum; sumVal++) {
            BigInteger sum = BigInteger.valueOf(sumVal);
            BigInteger square = sum.multiply(sum);
            
            // Skip if square has more than n digits
            if (square.compareTo(powersOf10[n]) >= 0) {
                break;
            }
            
            String squareStr = square.toString();
            
            // Check all possible splits of the square to see if it's a 2025-number
            for (int i = 1; i < squareStr.length(); i++) {
                BigInteger a = new BigInteger(squareStr.substring(0, i));
                BigInteger b = new BigInteger(squareStr.substring(i));
                
                String aStr = a.toString();
                String bStr = b.toString();
                boolean zeroCase = !aStr.concat(bStr).equals(squareStr) || a.equals(BigInteger.ZERO) || b.equals(BigInteger.ZERO);
                
                // Check if a + b = sumVal
                if (a.add(b).equals(sum) && !zeroCase) {
                    foundNumbers.add(new Result(square, a, b, sumVal));
                    break;
                }
            }
        }
        
        return foundNumbers;
    }
    
    ///////////////////////////////////////////
    // Multithreaded version

    /**
     * Task for executing find2025NumbersInRange in a separate thread
     */
    static class FindNumbersTask implements Callable<List<Result>> {
        private final int startSum;
        private final int endSum;
        private final int n;
        
        public FindNumbersTask(int startSum, int endSum, int n) {
            this.startSum = startSum;
            this.endSum = endSum;
            this.n = n;
        }
        
        @Override
        public List<Result> call() {
            return find2025NumbersInRange(startSum, endSum, n);
        }
    }
    
    /**
     * Calculate T(n) using parallel processing for performance.
     */
    public static BigInteger calculateTParallel(int n) {
        // Maximum possible sum value that could produce a 2025-number with n digits
        int maxSumValue = (int) Math.sqrt(Math.pow(10, n)) + 1;
        
        // Determine number of threads to use
        int cpuCount = Runtime.getRuntime().availableProcessors();
        System.out.println("Using parallel processing with " + cpuCount + " CPU cores...");
        
        // Split the work into chunks based on CPU count
        int chunkSize = Math.max(1, maxSumValue / cpuCount);
        List<FindNumbersTask> tasks = new ArrayList<>();
        
        for (int i = 0; i < maxSumValue; i += chunkSize) {
            int start = i + 1;  // Start from 1 for the first chunk
            int end = Math.min(i + chunkSize, maxSumValue);
            if (start <= end) {  // Ensure valid range
                tasks.add(new FindNumbersTask(start, end, n));
            }
        }
        
        // Use ExecutorService to find 2025-numbers in parallel
        ExecutorService executor = Executors.newFixedThreadPool(cpuCount);
        List<Future<List<Result>>> futures;
        try {
            futures = executor.invokeAll(tasks);
        } catch (InterruptedException e) {
            System.err.println("Thread execution was interrupted: " + e.getMessage());
            executor.shutdown();
            return BigInteger.ZERO;
        }
        
        // Combine and sort all results
        List<Result> allResults = new ArrayList<>();
        for (Future<List<Result>> future : futures) {
            try {
                allResults.addAll(future.get());
            } catch (InterruptedException | ExecutionException e) {
                System.err.println("Error retrieving results: " + e.getMessage());
            }
        }
        
        executor.shutdown();
        
        // Sort the results
        Collections.sort(allResults);

        // Display results
        System.out.println("Found " + allResults.size() + " 2025-numbers with " + n + " digits or less");
        /* Uncomment to print all results
        for (Result result : allResults) {
            System.out.println(result.number + " = concat(" + result.a + ", " + result.b + ") = (" + 
                             result.a + " + " + result.b + ")² = " + result.sumVal + "²");
        }
        */
        
        // Calculate and return the sum
        BigInteger total = BigInteger.ZERO;
        for (Result result : allResults) {
            total = total.add(result.number);
        }
        return total;
    }
    
    /**
     * Calculate T(n) - decide whether to use parallel processing based on n.
     */
    public static BigInteger calculateT(int n) {
        if (n >= 10) {
            return calculateTParallel(n);
        } else {
            // For smaller n, use single-threaded approach
            List<Result> results = find2025NumbersInRange(1, (int) Math.sqrt(Math.pow(10, n)) + 1, n);
            
            // Display results
            System.out.println("Found " + results.size() + " 2025-numbers with " + n + " digits or less");
            /* Uncomment to print all results
            for (Result result : results) {
                System.out.println(result.number + " = concat(" + result.a + ", " + result.b + ") = (" + 
                                 result.a + " + " + result.b + ")² = " + result.sumVal + "²");
            }
            */
            
            // Calculate and return the sum
            BigInteger total = BigInteger.ZERO;
            for (Result result : results) {
                total = total.add(result.number);
            }
            return total;
        }
    }
    
    // End multithreaded version
    ///////////////////////////////////////////
        
    public static void main(String[] args) {
        long startTime = System.currentTimeMillis();
        
        int n;
        if (args.length > 0) {
            n = Integer.parseInt(args[0]);
            BigInteger result = calculateT(n);
            System.out.println("T(" + n + ") = " + result);
        } else {
            // Verify that T(4) = 5131
            n = 4;
            BigInteger result = calculateT(n);
            System.out.println("T(" + n + ") = " + result);
            if (result.equals(BigInteger.valueOf(5131))) {
                System.out.println("Verification successful: T(4) = 5131");
            } else {
                System.out.println("Verification failed: Expected T(4) = 5131, got " + result);
            }
        }
        
        long endTime = System.currentTimeMillis();
        System.out.println("Execution time: " + (endTime - startTime) / 1000.0 + " seconds");
    }
}
