import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.linear.MatrixUtils; // For identity matrix

import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;

public class DiceGameExpectedValue {

    // --- Configuration ---
    private static final int NUM_PLAYERS = 50; // N

    // Check if N is even
    static {
        if (NUM_PLAYERS % 2 != 0) {
            throw new IllegalArgumentException("Number of players must be even.");
        }
        if (NUM_PLAYERS < 2) {
             throw new IllegalArgumentException("Number of players must be at least 2.");
        }
    }

    // Dice roll probabilities
    private static final double P_LEFT = 1.0 / 6.0;  // Roll 1
    private static final double P_RIGHT = 1.0 / 6.0; // Roll 6
    private static final double P_KEEP = 4.0 / 6.0;  // Roll 2, 3, 4, 5

    // --- State Mapping ---
    private static final Map<Integer, Pair> indexToStateMap = new HashMap<>();
    private static final Map<Pair, Integer> stateToIndexMap = new HashMap<>();
    private static int transientStateCount = 0;

    private static class Pair {
        final int i;
        final int j;

        Pair(int i, int j) {
            this.i = i;
            this.j = j;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            Pair pair = (Pair) o;
            // Consider (i, j) and (j, i) the same for mapping?
            // The pseudocode implies they are distinct matrix indices if i != j.
            // Let's stick to the pseudocode's distinct mapping first.
            // If the problem implied (1,5) and (5,1) were identical states,
            // we'd need canonical ordering (e.g., always store min index first).
            // But the description treats the *dice* positions distinctly.
            return i == pair.i && j == pair.j;
        }

        @Override
        public int hashCode() {
            // Same logic as equals - treat (i,j) and (j,i) distinctly for now
            int result = Integer.hashCode(i);
            result = 31 * result + Integer.hashCode(j);
            return result;
        }

        @Override
        public String toString() {
            return "(" + i + ", " + j + ")";
        }
    }

    // Pre-compute mappings for efficiency
    private static void initializeMappings(int n) {
        indexToStateMap.clear();
        stateToIndexMap.clear();
        transientStateCount = 0;
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (i != j) { // Only map transient states
                    Pair state = new Pair(i, j);
                    indexToStateMap.put(transientStateCount, state);
                    stateToIndexMap.put(state, transientStateCount);
                    transientStateCount++;
                }
            }
        }
         if (transientStateCount != n * (n - 1)) {
             System.err.println("Warning: Transient state count mismatch. Expected: " + (n * (n-1)) + ", Found: " + transientStateCount);
         }
    }

    // Map state (i, j) to a matrix index
    // Returns -1 if it's an absorbing state (i == j)
    private static int stateToIndex(int i, int j, int n) {
        if (i == j) {
            return -1; // Absorbing state, not in Q
        }
        Pair state = new Pair(i, j);
        // Assumes initializeMappings has been called
        Integer index = stateToIndexMap.get(state);
        if (index == null) {
             // This should not happen if mappings are initialized correctly
             throw new IllegalStateException("State (" + i + "," + j + ") not found in mapping.");
        }
        return index;
    }

    // Map matrix index back to state (i, j) - useful for debugging
    private static Pair indexToState(int index, int n) {
         // Assumes initializeMappings has been called
         Pair state = indexToStateMap.get(index);
         if (state == null) {
              throw new IllegalArgumentException("Index " + index + " not found in mapping.");
         }
         return state;
    }


    // --- Matrix Construction ---
    private static RealMatrix buildQ(int n) {
        int size = n * (n - 1); // Number of transient states
        if (size != transientStateCount) {
             System.err.println("Warning: Size mismatch in buildQ. Expected: " + size + ", Found: " + transientStateCount);
             // Use the actual count from mapping initialization
             size = transientStateCount;
        }

        // Using Array2DRowRealMatrix - consider sparse matrix for very large N if memory/performance becomes an issue
        RealMatrix Q = new Array2DRowRealMatrix(size, size); // Initialized with zeros

        System.out.println("Building Q matrix (" + size + "x" + size + ")...");

        // Iterate through all transient source states using the pre-computed map
        for (int sourceIndex = 0; sourceIndex < size; sourceIndex++) {
            Pair sourceState = indexToState(sourceIndex, n);
            int i = sourceState.i;
            int j = sourceState.j;

            // Define potential next positions for player i
            int i_next_keep = i;
            int i_next_left = (i - 1 + n) % n;
            int i_next_right = (i + 1) % n;

            // Define potential next positions for player j
            int j_next_keep = j;
            int j_next_left = (j - 1 + n) % n;
            int j_next_right = (j + 1) % n;

            // Define potential next states and probabilities
            int[] i_next_options = {i_next_keep, i_next_left, i_next_right};
            int[] j_next_options = {j_next_keep, j_next_left, j_next_right};
            double[] i_probs = {P_KEEP, P_LEFT, P_RIGHT};
            double[] j_probs = {P_KEEP, P_LEFT, P_RIGHT};

            // Calculate transitions for the 9 possible outcomes (3 for i * 3 for j)
            for (int i_opt = 0; i_opt < 3; i_opt++) {
                for (int j_opt = 0; j_opt < 3; j_opt++) {
                    int next_i = i_next_options[i_opt];
                    int next_j = j_next_options[j_opt];
                    double probability = i_probs[i_opt] * j_probs[j_opt];

                    // Check if the resulting state is transient (not absorbing)
                    if (next_i != next_j) {
                        int destIndex = stateToIndex(next_i, next_j, n);
                         if (destIndex != -1) { // Should always be true here due to check above
                            // The pseudocode implies setting, not adding. If multiple outcomes
                            // led to the *same* destination state, we would add probabilities.
                            // But each of the 9 outcomes defines a specific destination.
                            Q.setEntry(sourceIndex, destIndex, probability);
                         } else {
                             // Should not happen because of the next_i != next_j check
                             System.err.println("Error: Calculated destination index as -1 for a non-absorbing state transition.");
                         }
                    }
                    // If next_i == next_j, it's a transition to an absorbing state.
                    // These probabilities are implicitly captured in the R matrix,
                    // but we only need Q for the fundamental matrix calculation N=(I-Q)^-1.
                }
            }
             if (sourceIndex > 0 && sourceIndex % (size / 10) == 0) {
                 System.out.println(" ... built " + sourceIndex + "/" + size + " rows of Q.");
             }
        }
        System.out.println("Finished building Q matrix.");
        return Q;
    }

    // --- Fundamental Matrix Calculation ---
    // Solves (I-Q)X = 1, where 1 is a vector of ones.
    // The entries of X correspond to the expected number of steps.
    private static RealVector solveForExpectedSteps(RealMatrix Q) {
        int size = Q.getRowDimension();
        System.out.println("Preparing to solve linear system...");

        // Create Identity Matrix I
        RealMatrix I = MatrixUtils.createRealIdentityMatrix(size);
        System.out.println("Created Identity matrix I.");

        // Calculate I - Q
        RealMatrix IminusQ = I.subtract(Q);
        System.out.println("Calculated I - Q.");

        // Create the vector b (all ones)
        RealVector b = new ArrayRealVector(size, 1.0);
        System.out.println("Created vector b (ones).");

        // Solve (I - Q) * X = b for X
        System.out.println("Solving (I - Q) * X = b using LU Decomposition...");
        DecompositionSolver solver = new LUDecomposition(IminusQ).getSolver();

        if (!solver.isNonSingular()) {
             throw new RuntimeException("Matrix (I-Q) is singular, cannot solve the system. Check Q construction.");
        }

        RealVector X = solver.solve(b);
        System.out.println("Solved for X.");
        return X;
    }

    // --- Main Calculation ---
    public static double calculateExpectedTurns(int n) {
        if (n % 2 != 0 || n < 2) {
             throw new IllegalArgumentException("Number of players (n=" + n + ") must be even and at least 2.");
        }

        System.out.println("Initializing state mappings for N = " + n + "...");
        initializeMappings(n);
        System.out.println("Total transient states: " + transientStateCount);


        // Build the transition matrix Q
        RealMatrix Q = buildQ(n);

        // Calculate the expected number of steps vector X by solving (I-Q)X = b
        RealVector expectedStepsVector = solveForExpectedSteps(Q);

        // Find the index corresponding to the starting state (0, n/2)
        int start_i = 0;
        int start_j = n / 2;
        int startIndex = stateToIndex(start_i, start_j, n);

        if (startIndex == -1) {
            // This should only happen if n=2 and start is (0,1), which is transient.
             throw new IllegalStateException("Starting state ("+start_i+", "+start_j+") mapped incorrectly to absorbing state index.");
        }
         if (startIndex >= expectedStepsVector.getDimension()) {
             throw new IndexOutOfBoundsException("Start index " + startIndex + " is out of bounds for expected steps vector (size " + expectedStepsVector.getDimension() + ").");
         }

        // The expected number of turns starting from (0, n/2) is the value
        // in the expected steps vector X at the corresponding index.
        double expectedTurns = expectedStepsVector.getEntry(startIndex);

        return expectedTurns;
    }


    public static void main(String[] args) {
        System.out.println("Calculating expected turns for Dice Game with N = " + NUM_PLAYERS + " players.");
        long startTime = System.currentTimeMillis();

        try {
            double expectedTurns = calculateExpectedTurns(NUM_PLAYERS);

            long endTime = System.currentTimeMillis();
            long durationMillis = endTime - startTime;
            double durationSeconds = durationMillis / 1000.0;

            // Format output for readability
            DecimalFormat df = new DecimalFormat("#.##########"); // Aim for 10 decimal places

            System.out.println("\n----------------------------------------");
            System.out.println("Number of Players (N): " + NUM_PLAYERS);
            System.out.println("Initial State: Player 0 and Player " + (NUM_PLAYERS / 2));
            System.out.println("Expected Number of Turns: " + df.format(expectedTurns));
            System.out.println("Calculation Time: " + durationSeconds + " seconds");
            System.out.println("----------------------------------------");

        } catch (Exception e) {
            System.err.println("\nAn error occurred during calculation:");
            e.printStackTrace();
        }
    }
}
