import java.math.BigInteger;

public class NonLinearModular {

    // Modular exponentiation (long version, assumes result fits in long)
    public static long power(long base, long exp, long modulus) {
        BigInteger result = BigInteger.valueOf(base)
                                    .modPow(BigInteger.valueOf(exp), BigInteger.valueOf(modulus));
        return result.longValueExact(); // Throws exception if doesn't fit long
    }

    // Computes H(n) mod M iteratively
    public static long computeHmodM(long n, long baseIndex, long baseValueModM, long modulus) {
        if (n < baseIndex) {
            // Handle base cases before recurrence starts (e.g., H(1) if needed)
             if (n == 1 && baseIndex >=2) return 1L; // Assuming H(1)=1 based on image C(1)=1
            throw new IllegalArgumentException("n is smaller than base index " + baseIndex);
        }
        if (n == baseIndex) {
            return baseValueModM;
        }

        long currentHmodM = baseValueModM;
        long twentySevenModM = 27L % modulus;

        // Iterate from baseIndex + 1 up to n
        for (long i = baseIndex + 1; i <= n; i++) {
            long h_prev_cubed_mod_M = power(currentHmodM, 3L, modulus);
            currentHmodM = (twentySevenModM * h_prev_cubed_mod_M) % modulus;
            // Handle potential negative result from % if modulus is negative (not usually the case)
             if (currentHmodM < 0) {
                 currentHmodM += modulus;
             }
        }
        return currentHmodM;
    }

     public static void main(String[] args) {
         long M = 13L*13*13*13*13*13*13*13; // 13^8 = 302875106592253
         long N = 10000L;

         // Define base case for the recurrence H(n) = 27*H(n-1)^3
         // Option 1: Use H(1)=1 -> H(2)=27. Set baseIndex=1, baseValue=1
         // Option 2: Use H(2)=1 (from image C(2)=1) as separate base case. Set baseIndex=2, baseValue=1
         long baseIndex = 2;
         long baseValueModM = 1L; // H(2) mod M = 1 mod M = 1

         long h10000modM = computeHmodM(N, baseIndex, baseValueModM, M);
         System.out.println("H(10000) mod " + M + " = " + h10000modM);
         // NOTE: This value will likely NOT match 617720485 from the image because the recurrence is different.
     }
}
