import java.util.*;
import java.util.Arrays;
import com.google.common.graph.Graph;
import com.google.common.graph.GraphBuilder;
import com.google.common.graph.ImmutableGraph;
import com.google.common.graph.MutableGraph;
import com.google.common.graph.EndpointPair;

/** Figurate numbers using Guava:
 *
 * Find the sum of ordered set of six cyclic 
 * 4-digit numbers for which each polygonal type,
 * triangle, square, pentagonal, hexagonal,
 * heptagonal, octagonal, is represented
 * by a different number in the set.
 *
 * Approach:
 * Set up a graph, with each possible connection between 
 * prefixes and suffixes marked with an edge. 
 
 * Then we have 6 partitions in the graph.
 * 
 * We seek a cycle.
 */
public class GuavaFigurate { 

	public static final int NSHAPES = 8-3+1;

	public static void main(String[] args) { 
		solve();
	}

	public static void solve() { 
		MutableGraph<Node> cycle = buildGraph();
		System.out.println("Number of nodes: "+cycle.nodes().size());
		System.out.println("Number of edges: "+cycle.edges().size());

		// Now find a cycle
		CycleChecker cc = new CycleChecker(cycle,NSHAPES);
		Stack<Integer> foundCycle = cc.findCycles();
		int sum = 0;
		while(foundCycle!=null && !foundCycle.isEmpty()) {
			sum += foundCycle.pop();
		}
		System.out.println(sum);
	}





	/********************************
	 * Build the graph
	 * ******************************/

	/** Construct the graph containing all the shape numbers, with edges. */
	public static MutableGraph<Node> buildGraph() { 

		MutableGraph<Node> cycle = GraphBuilder.directed().build();

		// pancakes
		List<Map<Integer,List<Node>>> prefixMaps  = new ArrayList<>();
		List<Map<Integer,List<Node>>> postfixMaps = new ArrayList<>();

		//////////////////////////////////
		// Construct nodes

		// Loop over all shape groups
		for(int s=3; s<3+NSHAPES; s++) { 
			Set<Integer> shapeNumbers = getAll(s);

			// Maintain prefix/postfix map separately for each shape 
			// to ease edge-building
			Map<Integer,List<Node>> prefixMap  = new HashMap<>();
			Map<Integer,List<Node>> postfixMap = new HashMap<>();

			for(Integer sh : shapeNumbers) {
				int prefix  = sh/100;
				int postfix = sh%100;
				// postfix might be a party pooper
				if(postfix!=0) { 
					Node node = new Node(sh,s);

					// Add ourselves to prefix and postfix maps
					if(prefixMap.keySet().contains(prefix)) {
						List<Node> nodelist = prefixMap.get(prefix);
						nodelist.add(node);
					} else {
						List<Node> nodelist = new ArrayList<Node>();
						nodelist.add(node);
						prefixMap.put(prefix,nodelist);
					}

					if(postfixMap.keySet().contains(postfix)) {
						List<Node> nodelist = postfixMap.get(postfix);
						nodelist.add(node);
					} else {
						List<Node> nodelist = new ArrayList<Node>();
						nodelist.add(node);
						postfixMap.put(postfix,nodelist);
					}
				}
			}

			prefixMaps.add(prefixMap);
			postfixMaps.add(postfixMap);
		}

		//////////////////////////////////
		// Link nodes

		// Loop over all shape groups
		// and link the nodes together
		for(int mapsIndex=0; mapsIndex < prefixMaps.size(); mapsIndex++) { 

			int keycount;
			int actualkeycount;
			Map<Integer,List<Node>> prefixMap = prefixMaps.get(mapsIndex);
			Map<Integer,List<Node>> postfixMap = postfixMaps.get(mapsIndex);

			// Loop over every prefix in this shape group
			// Connect it with postfixes in all other groups 
			keycount = 0;
			actualkeycount = 0;
			for(Integer key : prefixMap.keySet()) { 

				// Loop over all other postfix maps
				for(int otherMapsIndex=0; otherMapsIndex < postfixMaps.size(); otherMapsIndex++) { 
					if(otherMapsIndex==mapsIndex) { 
						// same shape, different day
						continue;
					}
					List<Node> sinks = postfixMaps.get(otherMapsIndex).get(key);
					if(sinks!=null) { 
						List<Node> sources = prefixMaps.get(mapsIndex).get(key);
						for(Node source : sources) {
							for(Node sink : sinks) { 
								cycle.putEdge(source, sink);
								keycount++;
							}
						}
					}
				}
			}

			
			/////////// Now do the whole thing over again, 
			/////////// but matching up postfixes with prefixes. 
			/////////keycount = 0;
			/////////actualkeycount = 0;
			/////////for(Integer key : postfixMap.keySet()) { 

			/////////	// Loop over all other postfix maps
			/////////	for(int otherMapsIndex=0; otherMapsIndex < prefixMaps.size(); otherMapsIndex++) { 
			/////////		if(otherMapsIndex==mapsIndex) { 
			/////////			// same shape, different day
			/////////			continue;
			/////////		}
			/////////		List<Node> sources = prefixMaps.get(otherMapsIndex).get(key);
			/////////		if(sources!=null) { 
			/////////			List<Node> sinks = postfixMaps.get(mapsIndex).get(key);
			/////////			for(Node source : sources) {
			/////////				for(Node sink : sinks) { 
			/////////					cycle.putEdge(source, sink);
			/////////					keycount++;
			/////////				}
			/////////			}
			/////////		}
			/////////	}
			/////////}
		}

		return cycle;
	}


	/*******************************
	 * Shape function and inverse
	 * (used to construct graph)
	 * *****************************/

	/** Compute the shape function for an s-sided shape. */
	public static int shapeFunction(int n, int s) { 
		int a = s - 2;
		int b = s - 4;
		return (n*(a*n-b))/2;
	}

	/** Compute the inverse shape function for an s-sided shape. */
	public static double inverseShapeFunction(int limit, int s) { 
		int a = s - 2;
		int b = s - 4;
		return (b + Math.sqrt(b*b + 8*a*limit))/(2.0*a);
	}

	/** Get all 4-digit numbers corresponding to an s-sided shape. */
	public static Set<Integer> getAll(int s) { 
		int floor = 1000;
		int ceil = 9999;
		int start = (int)(Math.ceil(inverseShapeFunction(floor,s)));
		int end = (int)(Math.floor(inverseShapeFunction(ceil,s)));
		Set<Integer> result = new HashSet<>();
		for(int k=start; k<=end; k++) { 
			int num = shapeFunction(k,s);
			// anything with second to last digit of 0 requires 3-digit numbers.
			if( (num/10)%10 != 0) {
				result.add(num);
			}
		}
		return result;
	}



}



/********************************
 * Cycle-Checking
 * ******************************/

/** Class to check a directed graph for cycles. */
class CycleChecker { 
	Stack<Integer> cycle;
	MutableGraph<Node> graph;
	public static int NSHAPES;

	/** Constructor takes a graph and stores it. */
	public CycleChecker(MutableGraph<Node> graph, int nshapes) { 
		this.graph = graph;
		NSHAPES = nshapes;
	}

	/** Depth-first search to find nodes composing a cycle in this graph. */
	public Stack<Integer> findCycles() {
		Set<Node> vertices = graph.nodes();
		Set<Node> solutionSet = new HashSet<Node>();
		boolean[] shapes = new boolean[9];
		for(Node vertex : vertices) { 
			if(!vertex.visited) {

				vertex.first();
				shapes[vertex.shape] = true;

				dfs(vertex, null, solutionSet, shapes, 0);
			}
		}
		return this.cycle;
	}

	protected void dfs(Node v, Node previous, Set<Node> solutionSet, boolean[] shapes, int depth) {
		if(depth==NSHAPES) { 

			// Base case
			for(Node w : graph.adjacentNodes(v)) { 
				if(w.first && !previous.first) { 
					// Completed a circuit.
					System.out.println("Found solution:");
					System.out.println(solutionSet);
				}
			}
			// Nope.
			return;

		} else {

			// Recursive case
			for(Node w : graph.adjacentNodes(v)) { 

				for(int i = 0; i < depth; i++ ) {
					System.out.print("    ");
				}
				System.out.println("Node "+v+" child "+w);

				// if we don't already have something from this partition
				if(!shapes[w.shape]) {
					// If we haven't already visited this node
					if(!w.visited) { 
						solutionSet.add(w);
						shapes[w.shape] = true;
						w.visit();

						dfs(w, v, solutionSet, shapes, depth+1);

						solutionSet.remove(w);
						shapes[w.shape] = false;
						w.unvisit();
					}
				}
			}
			return;
		}
	}
}

/** Node class */
class Node {
	public int value;
	public int shape;
	public int prefix;
	public int postfix;
	public boolean visited;
	public boolean first;
	public Node(int n, int shapeFactor){
		value = n;
		shape = shapeFactor;
		prefix = n/100;
		postfix = n%100;
		visited = false;
	}
	public String toString() { 
		StringBuffer sb = new StringBuffer();
		sb.append(value);
		sb.append(" [");
		sb.append(shape);
		sb.append("] ");
		return sb.toString();
	}
	public void first() { 
		first = true;
	}
	public void visit() { 
		visited = true;
	}
	public void unvisit() { 
		visited = false;
	}
}
