import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.List;
import java.util.Queue;
import java.util.LinkedList;
import java.util.ArrayList;

/** Prime number generator class
 * implemented using BigInteger.
 *
 * Uses a segmented sieve of Eratosthenes
 * algorithm.
 * */
public class PrimeGeneratorBig { 

	/**
	 * Prime Generator class implementation.
	 */

	private Queue<BigInteger> q;
	private BigInteger max;
	private BigInteger lo, hi;
	private int rootmax;

	private int cycle;

	private static final BigInteger TWO = new BigInteger("2");
	private static final BigInteger NEGATIVEONE = new BigInteger("-1");
	private static final double LOG2 = Math.log(2.0);

	public PrimeGeneratorBig(BigInteger max) {
		this.max = max;
		this.rootmax = approxSqrt(this.max);
		// Making rootmax even makes life easier.
		if(rootmax%2!=0) {
			rootmax++;
		}

		this.q = new LinkedList<BigInteger>();

		this.lo = BigInteger.ONE;
		this.hi = BigInteger.ONE;

		// Look ahead. This enables hasNext() check.
		repopulateQueue();
	}

	public BigInteger next() { 
		BigInteger result = q.remove();

		// Look ahead. This enables hasNext() check.
		if(q.isEmpty()) {
			repopulateQueue();
		}

		return result;
	}




	/** Repopulate queue with prime number starting at lo and going to hi. */
	public void repopulateQueue() { 

		int ACCORDION = (int)(Math.round(Math.sqrt(rootmax)));

		// Value of ACCORDION*sqrt(MAX) is the stride 
		// of our segmented Sieve of Eratosthenes algorithm.

		if(lo.equals(BigInteger.ONE)) {
			this.cycle = 1;
			// First time through,
			// use normal Sieve of Eratosthenes.

			// Add everything up to sqrt(MAX).
			// This ensures we don't call segmented Sieve of Eratosthenes 
			// when low < sqrt(hi), because we already begin 
			// with low above sqrt(MAX).
			q.addAll( primeSieve(rootmax) );

			// set new lo and hi.
			// lo must be even. 
			// only need to do this once.
			lo = BigInteger.valueOf(rootmax);
			hi = BigInteger.valueOf(rootmax + ACCORDION*rootmax);

		} else {
			this.cycle++;

			// Every other time through,
			// use segmented Sieve of Eratosthenes.
			q.addAll( primesRange(lo, hi) );

			lo = hi;
			hi = hi.add( BigInteger.valueOf( ACCORDION*rootmax ));
		}
	}





	/** Find all prime numbers between left and right as longs. 
	 * */
	public static List<Long> intPrimesRange(long left, long right) { 

		// Return this
		ArrayList<Long> result = new ArrayList<Long>();

		// Ensure that we get precisely the correct range of primes.
		int delta;
		delta = (int)(right-left);
		delta /= 2;

		boolean[] sieve = new boolean[delta-1];

		// could make these longs... if sqrt(right) < 10^18, which, obviously.
		// could make these ints... if sqrt(right) < 10^9, which... also has to be.
		// these compromises are needed to keep the class working, 
		// but cripple it for other applications. 
		// 
		// ...but let future self deal with those and hack away at this
		// when we actually get there. For now, just get this done.
		//
		List<Integer> Ps = intPrimeSieve( (int)(Math.sqrt(right)) );

		// Skip 2
		Ps.remove(Ps.indexOf(2));

		// Create Qs starting index vector
		List<Integer> Qs = new ArrayList<Integer>();
		for(int i=0; i<Ps.size(); i++ ) {
			Qs.add( intqInit(left, Ps.get(i)) );
		}

		// Loop over each cluster
		while(left<right){

			// Assume everybody prime
			for(int k=0; k<(delta-1); k++) { 
				sieve[k] = true;
			}

			// Cross out multiple of each prime
			for(int i=0; i<Ps.size(); i++) {
				int p = Ps.get(i).intValue();
				int q = Qs.get(i);

				for(int j=q; j<(delta-1); j+=p) { 
					sieve[j] = false;
				}
			}

			// Reset for next cluster
			for(int k=0; k<Ps.size(); k++ ) {
				Qs.set(k, qReset(Ps.get(k), Qs.get(k), delta) );
			}

			// Have knocked out non-primes, so gather the primes
			long t = left + 1;
			for(int i=0; i<delta-1; i++) { 
				// t ends at right
				if(sieve[i]) { 
					result.add( new Long(t) );
				}
				t += 2;
			}
			left += 2*delta; 
		}
		return result;
	}

	protected static int intqInit(long left, int pbig) {

		//// Qk = -(1/2) (L + 1 + Pk) mod Pk
		//modOfThis = left.add(pbig).add(BigInteger.ONE).multiply(NEGATIVEONE).divide(TWO);
		//BigInteger j = BigInteger.ONE;
		//while(modOfThis.compareTo(BigInteger.ONE)<0) {
		//	modOfThis = modOfThis.add( pbig.multiply(j) );
		//	// If our maximum gets bigger than 10^11, 
		//	// j will need to be a BigInteger.
		//	j = j.multiply(TWO);
		//}

		// modOfThis must be made positive.
		long modOfThis = (left*pbig - left + pbig + 1)/2;
		int result = (int)(modOfThis%((long)pbig));
		return result;
	}








	/** Find all prime numbers between left and right. 
	 * This requires left is bigger than sqrt(right), since that is where it begins.
	 * (User is expected to take care of everything up to at least sqrt(right) using 
	 * regular Sieve of Eratosthenes algorithm.)
	 * */
	public static List<BigInteger> primesRange(BigInteger left, BigInteger right) { 

		// Return this
		ArrayList<BigInteger> result = new ArrayList<BigInteger>();

		// Ensure that we get precisely the correct range of primes.
		int delta;
		delta = right.subtract(left).intValue();
		delta /= 2;

		boolean[] sieve = new boolean[delta-1];

		// could make these longs... if sqrt(right) < 10^18, which, obviously.
		// could make these ints... if sqrt(right) < 10^9, which... also has to be.
		// these compromises are needed to keep the class working, 
		// but cripple it for other applications. 
		// 
		// ...but let future self deal with those and hack away at this
		// when we actually get there. For now, just get this done.
		//
		List<BigInteger> Ps = primeSieve( approxSqrt(right) );

		// Skip 2
		for(BigInteger p : Ps) { 
			if(p.equals(new BigInteger("2"))) { 
				Ps.remove(p);
				break;
			}
		}

		// Create Qs starting index vector
		List<Integer> Qs = new ArrayList<Integer>();
		for(int i=0; i<Ps.size(); i++ ) {
			Qs.add( qInit(left, Ps.get(i)) );
		}

		// Loop over each cluster
		while(left.compareTo(right)<0) { 

			// Assume everybody prime
			for(int k=0; k<(delta-1); k++) { 
				sieve[k] = true;
			}

			// Cross out multiple of each prime
			for(int i=0; i<Ps.size(); i++) {
				int p = Ps.get(i).intValue();
				int q = Qs.get(i);

				for(int j=q; j<(delta-1); j+=p) { 
					sieve[j] = false;
				}
			}

			// Reset for next cluster
			for(int k=0; k<Ps.size(); k++ ) {
				Qs.set(k, qReset(Ps.get(k).intValue(), Qs.get(k).intValue(), delta) );
			}

			// Have knocked out non-primes, so gather the primes
			BigInteger t = left.add(BigInteger.ONE);
			for(int i=0; i<delta-1; i++) { 
				// t ends at right
				if(sieve[i]) { 
					result.add( t );
				}
				t = t.add(TWO);
			}
			left = left.add(BigInteger.valueOf(2*delta));
		}
		return result;
	}

	protected static int qInit(BigInteger left, BigInteger pbig) {
		BigInteger modOfThis;

		//// Qk = -(1/2) (L + 1 + Pk) mod Pk
		//modOfThis = left.add(pbig).add(BigInteger.ONE).multiply(NEGATIVEONE).divide(TWO);
		//BigInteger j = BigInteger.ONE;
		//while(modOfThis.compareTo(BigInteger.ONE)<0) {
		//	modOfThis = modOfThis.add( pbig.multiply(j) );
		//	// If our maximum gets bigger than 10^11, 
		//	// j will need to be a BigInteger.
		//	j = j.multiply(TWO);
		//}

		// modOfThis must be made positive.
		modOfThis = left.subtract(pbig).pow(2).add(BigInteger.ONE).divide(TWO).mod(pbig);

		BigInteger modint = modOfThis.mod(pbig);
		int result = Integer.parseInt(modint.toString());
		return result;
	}

	protected static int qReset(int p, int q, int delta) {
		int result = (q-delta)%p;
		int j = 1;
		while(result < 0) { 
			result += j*p;
		}
		return result;
	}





	/** Find all prime numbers below n. */
	public static List<BigInteger> primeSieve(int n) { 

        // initially assume all integers are prime 
        boolean[] isPrime = new boolean[n+1];
        for (int i = 2; i <= n; i++) {
            isPrime[i] = true;
        }

        // mark non-primes <= n using Sieve of Eratosthenes
        for (int factor = 2; factor*factor <= n; factor++) {

            // if factor is prime, then mark multiples of factor as nonprime
            // suffices to consider mutiples factor, factor+1, ...,  n/factor
            if (isPrime[factor]) {
                for (int j = factor; factor*j <= n; j++) {
                    isPrime[factor*j] = false;
                }
            }
        }

		// Add primes to list
		ArrayList<BigInteger> primes = new ArrayList<BigInteger>();
		int nprimes = 0;
		for(int k=2; k<=n; k++) {
			if(isPrime[k]) { 
				primes.add( BigInteger.valueOf(k) );
				nprimes++;
			}
		}

		return primes;
	}


	/** Find all prime numbers below n.
	 *
	 * Return then as Integers.
	 */
	public static List<Integer> intPrimeSieve(int n) { 

        // initially assume all integers are prime 
        boolean[] isPrime = new boolean[n+1];
        for (int i = 2; i <= n; i++) {
            isPrime[i] = true;
        }

        // mark non-primes <= n using Sieve of Eratosthenes
        for (int factor = 2; factor*factor <= n; factor++) {

            // if factor is prime, then mark multiples of factor as nonprime
            // suffices to consider mutiples factor, factor+1, ...,  n/factor
            if (isPrime[factor]) {
                for (int j = factor; factor*j <= n; j++) {
                    isPrime[factor*j] = false;
                }
            }
        }

		// Add primes to list
		ArrayList<Integer> primes = new ArrayList<Integer>();
		int nprimes = 0;
		for(int k=2; k<=n; k++) {
			if(isPrime[k]) { 
				primes.add( new Integer(k) );
				nprimes++;
			}
		}

		return primes;
	}



	/** The prime generator is agnostic to the maximum;
	 * we use hasNext to determine when to stop. 
	 * Change this to true for an infinite prime generator.
	 * */
	public boolean hasNext() { 
		// This is possible because we always ensure there is something in the queue
		// before we finish operations on the queue.
		if(q.peek().compareTo(max)>0){ 
			return false;
		} else {
			return true;
		}
	}


	/**
	 * Computes the natural logarithm of a BigInteger. Works for really big
	 * integers (practically unlimited)
	 * 
	 * @param val Argument, positive integer
	 * @return Natural logarithm, as in <tt>Math.log()</tt>
	 */
	public static double logBigInteger(BigInteger val) {
	    int blex = val.bitLength() - 1022; // any value in 60..1023 is ok
	    if (blex > 0)
	        val = val.shiftRight(blex);
	    double res = Math.log(val.doubleValue());
	    return blex > 0 ? res + blex * LOG2 : res;
	}

	/**
	 * Same as Math.exp() but returns a BigInteger. Oriented towards big numbers.
	 * Works even for outputs that exceed the double range
	 *
	 * @param b Should be a large positive value
	 * @return The value of e (base of the natural logarithms) raised to the power b
	 */
	public static BigInteger bigexp(double b) {
	    if( Double.isNaN(b) || Double.isInfinite(b) ) 
	        throw new IllegalArgumentException("Infinite or negative values not accepted: " + b);
	    // e^b = e^(b2+c) = e^b2 2^t with e^c = 2^t 
	    double bc = 680.0;
	    if( b < bc ) 
	        return BigDecimal.valueOf(Math.exp(b)).setScale(0, BigDecimal.ROUND_HALF_EVEN).toBigInteger();
	    double b2 = bc;
	    double c = b - bc;
	    int t = (int) Math.ceil(c / LOG2);
	    c = t * LOG2;
	    b2 = b - c;
	    BigInteger v = BigDecimal.valueOf(Math.exp(b2)).setScale(0, BigDecimal.ROUND_HALF_EVEN).toBigInteger();
	    return v.shiftLeft(t);
	}
	
	public static BigInteger bigpow(double a, double b) {
	    return bigexp(b * Math.log(a));
	}

	public static int approxSqrt(BigInteger n) { 
		return (int)(Math.round(Math.exp( logBigInteger(n)/2 )));
	}

}


